% get interface to COM
h = actxserver('RPI_IOBoard.IOBoard')
% h.interfaces % show interfaces


% test core functions
iob = h.invoke('ICoreFunctions')
%iob.invoke % show functions for this interface
%iob.get    % show properties for this interface

num_devices = iob.getNumDevices()
if (num_devices < 1)
    error 'No boards connected'
    return
end

% connect to the board
iob.openDevice(1)

% get some information about the board
iob.FirmwareVersion
iob.HardwareRevision
iob.identifyDevice();  % flash blue LED on boards which support it
devInfo = iob.attachedDeviceInfo

% only applies to red1 and red2 boards (blue always returns true)
if (~iob.isCalibrated())
    error 'Please use Mobile Studio to calibrate this board.'
end


% test digital i/o
dig_io = h.invoke('IDigitalIO');
%dig_io.invoke % show functions for this interface
%dig_io.get    % show properties for this interface

% these assume lines 1 and 2 are wired together
dig_io.setDigitalDirection(1, 0);
dig_io.setDigitalDirection(2, 1);
dig_io.setDigitalState(1, 1);
[dig_io.getDigitalState(2) 1]
dig_io.setDigitalState(1, 0);
[dig_io.getDigitalState(2) 0]

num_lines = length(dig_io.getDigitalDirections());
feature('COM_SafeArraySingleDim', 1);
dig_io.setDigitalDirections(ones (num_lines, 1, 'int32'));
dig_io.getDigitalDirections()
dig_io.setDigitalDirections(zeros(num_lines, 1, 'int32'));
dig_io.getDigitalDirections()


% test PWMs (red2 boards only)
if strcmp(devInfo(2), '2100')
    disp 'testing PWMs'
    pwm = h.invoke('IPWM')
    %pwm.invoke % show functions for this interface
    %pwm.get    % show properties for this interface

    pwm.set_Period(1, 0.02);
    pwm.set_Period(2, 0.02);
    
    % setting offset locks duty cycle at 50%, so can only use one or the other
    usingDutyCycle = true;
    if usingDutyCycle
        pwm.set_DutyCycle(1, 0.25);
        pwm.set_DutyCycle(2, 0.75);
    else
        % setting offset=0 not actually valid, but this sets it to minimum
        pwm.set_PWMOffset(1, 0);
        pwm.set_PWMOffset(2, 0.003);
    end
    
    pwm.StartAllPWMChannelsSynchronously();
end


% test analog i/o (red1 boards only)
% interface is available for red2, but just wraps AnalogOut/AnalogStream
% (channel types are also different for red2; see getAnalogChannelTypes())
ana_io = h.invoke('IAnalogIO');
%ana_io.invoke % show functions for this interface
%ana_io.get    % show properties for this interface

% see which analog channel types we have available
channelTypes = iob.getAnalogChannelTypes(1);

% try setting ADC1 and reading it back in at DAC1
% (if we have those channel types on this board)
have_ADC1 = length(strmatch('ADC1', channelTypes));
have_DAC1 = length(strmatch('DAC1', channelTypes));
if (have_ADC1 && have_DAC1)
    % assuming ADC1 is wired to DAC1

    % set ADC1 to 2.5V
    %
    % (setAnalogOutputValue() may not be available on a given board;
    %  it returns true or false to indicate success or failure)
    ana_io.setAnalogOutputValue(1, 2.5);
    
    % read voltage back in through ADC1 using different gain settings
    %
    % (setAnalogInputVoltageRange() returns range that was actually set,
    %  which may be always equal to 1 if feature is not available for a
    %  particular board)
    ana_io.setAnalogInputVoltageRange('ADC1', 5);
	% (getAnalogValue() may not be available on a given board;
	%  it throws an exception if not implemented)
    ana_io.getAnalogValue('ADC1')
    ana_io.setAnalogInputVoltageRange('ADC1', 10);
    ana_io.getAnalogValue('ADC1')
end


% test function generators
fg = h.invoke('IFuncGen');
%fg.invoke % show functions for this interface
%fg.get    % show properties for this interface

fg_chan = 1;
fg.set_Waveform(fg_chan, 'Sine');
fg.set_Frequency(fg_chan, 1000);
fg.set_PkPk(fg_chan, 1.0);
fg.set_Offset(fg_chan, 0);


% test analog out streams (red2 boards only)
if strcmp(devInfo(2), '2100')
    ana_out = h.invoke('IAnalogOut');
    %ana_out.invoke % show functions for this interface
    %ana_out.get    % show properties for this interface

    awg_chan = 2;
    % this gives the list of rates available in Mobile Studio, but other 
    % (though not all) rates are also possible
    rates = ana_out.get_AvailableSampleRates(awg_chan);
    rate3 = cell2mat(rates(3));  % get 3rd entry from list and convert to string
    ana_out.set_SampleRateString(awg_chan, rate3)
    [rate3 ', ' ana_out.get_SampleRateString(awg_chan) ', ' num2str(ana_out.get_SampleRate(awg_chan))]

    % we used simple function generator interface above for channel 1, but
    % here we build our own signal for channel 2 using analog out interface
    theta = 0:0.1:2*pi;
    data = sin(theta) + sin(2*theta);
    ana_out.set_SampleRate(awg_chan, 440*length(data));
    ana_out.get_SampleRateString(awg_chan)
    ana_out.set_Looping(awg_chan, true);
    ana_out.StopAllChannelsSynchronously()
    ana_out.WriteData(awg_chan, data')  % always stop AWGs before writing data!
    ana_out.StartAllChannelsSynchronously()
else
    % use func gen instead
    fg_chan = 2;
    fg.set_Waveform(fg_chan, 'Sine');
    fg.set_Frequency(fg_chan, 2000);
    fg.set_PkPk(fg_chan, 1.0);
    fg.set_Offset(fg_chan, 0);
end


% test Audio (red2 boards only)
if strcmp(devInfo(2), '2100')
    audio = h.invoke('IAudioOut');
    %audio.invoke % show functions for this interface
    %audio.get    % show properties for this interface
    
    audio.Enabled = true;
    audio.HeadphonesConnected
    % SourceListLeft and SourceListRight give options for these
    audio.SourceLeft  = 'AWG 1';
    audio.SourceRight = 'AWG 2';
    % Volume_options() gives options for these
    % can also set/get VolumeLeftNum and VolumeRightNum (0-255)
    audio.VolumeLeft  = 'Med';
    audio.VolumeRight = 'Med';
end


% test analog input streams
ana_str = h.invoke('IAnalogStream');
%ana_str.invoke % show functions for this interface
%ana_str.get    % show properties for this interface

have_FUNC1 = length(strmatch('FUNC1', channelTypes));
have_FG1   = length(strmatch('FG1', channelTypes));
have_AWG1  = length(strmatch('AWG1', channelTypes));
if (have_FG1 || have_FUNC1 || have_AWG1)
    if (have_FUNC1)
		% channel names on red1
        chType1 = 'FUNC1';
        chType2 = 'FUNC2';
    end
    if (have_FG1)
		% channel names on blue
        chType1 = 'FG1';
        chType2 = 'FG2';
    end
    if (have_AWG1)
		% channel names on red2
        chType1 = 'AWG1';
        chType2 = 'AWG2';
    end
 
	feature('COM_SafeArraySingleDim', 1);

	ana_str.Averaging = 1;
	ana_str.ChannelCount = 2;          % only has an effect on red1 boards
	ana_str.CollectionMode = 'Stream'; % only has an effect on red1 boards
    % these two are set automatically when ana_str.TimeRange is set
	% ana_str.PointsPerChannel = ...
    % ana_str.SamplingDiv = 1;
	ana_str.TimeRange = [0.0; 0.01];
	ana_str.TriggerChannel = 1;
	ana_str.TriggerLevel = 0.0;
	ana_str.TriggerMode = 'Auto';
	ana_str.TriggerSlope = 'Rising';
    % for 'External', set TriggerChannel = # of dig i/o line to use
	ana_str.TriggerSource = 'Internal';

	% both channels must be initialized, even if only reading from one of them
	ana_str.set_Coupling(1, 'DC');
	ana_str.set_Coupling(2, 'DC');
 	ana_str.set_DisplayVerticalRange(1, [-4.0; 4.0]);
 	ana_str.set_DisplayVerticalRange(2, [-4.0; 4.0]);
	ana_str.set_InputChannel(1, chType1);
    ana_str.set_InputChannel(2, chType2);
    ana_str.Start()
    
    % these depend on DisplayVerticalRange and InputChannel settings
    % -1 for not set or not applicable (e.g., AWG internal loopback)
    ana_str.get_InputResistance(1)
    ana_str.get_InputResistance(2)

    x = ana_str.TimeRange(1):ana_str.DataRatePeriod:ana_str.TimeRange(2);
    disp 'press <control>-c to break out of loop'
    while(1)
        analogData1 = ana_str.GetData(1);
        analogData2 = ana_str.GetData(2);
        if (~isempty(analogData1) && ~isempty(analogData2))
            plot(x, analogData1, x, analogData2)
            drawnow
        end
    end
end


% close device
%iob.closeDevice()
